# -*- coding: utf-8 -*-

import base64
import logging
import os
import tempfile
import unittest
import uuid

import clarifai.rest
from clarifai.rest import ClarifaiApp, Geo, GeoBox, GeoLimit, GeoPoint

urls = [
    "https://samples.clarifai.com/metro-north.jpg",
    "https://samples.clarifai.com/wedding.jpg",
    "https://samples.clarifai.com/facebook.png",
    "https://samples.clarifai.com/dog.tiff",
    "https://samples.clarifai.com/penguin.bmp",
]


class TestSearch(unittest.TestCase):
  _multiprocess_can_split_ = True
  to_cleanup = []

  @classmethod
  def setUpClass(cls):
    cls.app = ClarifaiApp(log_level=logging.WARN)

  @classmethod
  def tearDownClass(cls):
    """ Cleanup """

  def test_search_annotated_concepts(self):
    """ test search by annotated concepts only """

    self.app.inputs.search_by_annotated_concepts(concept='cat')
    self.app.inputs.search_by_annotated_concepts(concepts=['cat'])
    self.app.inputs.search_by_annotated_concepts(concept_id='ai_mFqxrph2')
    self.app.inputs.search_by_annotated_concepts(concept_ids=['ai_mFqxrph2'])

    # go with unicode string
    self.app.inputs.search_by_annotated_concepts(concept=u'cat')
    self.app.inputs.search_by_annotated_concepts(concepts=[u'cat'])
    self.app.inputs.search_by_annotated_concepts(concept_id=u'ai_mFqxrph2')
    self.app.inputs.search_by_annotated_concepts(concept_ids=[u'ai_mFqxrph2'])

    # search with NOT
    self.app.inputs.search_by_annotated_concepts(concept='cat', value=False)
    self.app.inputs.search_by_annotated_concepts(concepts=['cat', 'dog'], values=[False, False])

  def test_search_predicted_concepts(self):
    """ test search by predicted concepts only """

    # upload some cat and dog
    img1 = self.app.inputs.create_image_from_url(urls[0], allow_duplicate_url=True)

    self.app.inputs.search_by_predicted_concepts(concept='train')
    self.app.inputs.search_by_predicted_concepts(concepts=['train'])
    self.app.inputs.search_by_predicted_concepts(concept_id='ai_mFqxrph2')
    self.app.inputs.search_by_predicted_concepts(concept_ids=['ai_mFqxrph2'])

    # go with unicode string
    self.app.inputs.search_by_predicted_concepts(concept=u'train')
    self.app.inputs.search_by_predicted_concepts(concepts=[u'train'])
    self.app.inputs.search_by_predicted_concepts(concept_id=u'ai_mFqxrph2')
    self.app.inputs.search_by_predicted_concepts(concept_ids=[u'ai_mFqxrph2'])

    # search with NOT
    self.app.inputs.search_by_predicted_concepts(concept='train', value=False)
    self.app.inputs.search_by_predicted_concepts(
        concepts=['train', 'railway'], values=[False, False])
    self.app.inputs.search_by_predicted_concepts(
        concepts=['train', 'railway'], values=[True, False])
    self.app.inputs.search_by_predicted_concepts(
        concepts=['train', 'railway'], values=[False, True])

    # clean up
    self.app.inputs.delete(img1.input_id)

  def test_search_predicted_concepts_multi_lang(self):
    """ test search by predicted concepts with non-English """

    # upload the metro north
    img1 = self.app.inputs.create_image_from_url(urls[0], allow_duplicate_url=True)

    # search in simplified Chinese
    imgs = self.app.inputs.search_by_predicted_concepts(concept=u'铁路列车', lang='zh')
    # disable this until citus migration fully finishes
    # self.assertGreaterEqual(len(list(imgs)), 1)

    imgs = self.app.inputs.search_by_predicted_concepts(concepts=[u'铁路'], lang='zh')
    # disable this until citus migration fully finishes
    # self.assertGreaterEqual(len(list(imgs)), 1)

    # search in Japanese
    imgs = self.app.inputs.search_by_predicted_concepts(concept=u'列車', lang='ja')
    # disable this until citus migration fully finishes
    # self.assertGreaterEqual(len(list(imgs)), 1)

    imgs = self.app.inputs.search_by_predicted_concepts(concepts=[u'鉄道'], lang='ja')
    # disable this until citus migration fully finishes
    # self.assertGreaterEqual(len(list(imgs)), 1)

    # clean up
    self.app.inputs.delete(img1.input_id)

  def test_search_by_image_url(self):
    """ test search by image url """

    # search by image url
    self.app.inputs.search_by_image(url=urls[0])

    # search by image url with leading space
    self.app.inputs.search_by_image(url=' ' + urls[0])

    # search by image url with trailing space
    self.app.inputs.search_by_image(url=urls[0] + ' ')

  def test_search_by_image_id(self):
    """ test search by image id """

    # search by exising input id
    image = self.app.inputs.create_image_from_url(urls[0], allow_duplicate_url=True)

    self.app.inputs.search_by_image(image_id=image.input_id)

    # clean up
    self.app.inputs.delete(image.input_id)

  def test_search_by_image_bytes(self):
    """ test search by image bytes """

    # search by raw bytes
    data = self.app.api.session.get(urls[0]).content
    self.app.inputs.search_by_image(imgbytes=data)

  def test_search_by_image_base64(self):
    """ test search by image base64 bytes """

    # search by base64 bytes
    data = self.app.api.session.get(urls[0]).content
    base64_data = base64.b64encode(data)
    self.app.inputs.search_by_image(base64bytes=base64_data)

  def test_search_by_image_file(self):
    """ test search by image filename and fileobj"""

    data = self.app.api.session.get(urls[0]).content
    f = tempfile.NamedTemporaryFile(delete=False)
    filename = f.name
    f.write(data)
    f.close()

    self.app.inputs.search_by_image(filename=filename)

    fio = open(filename, 'rb')
    self.app.inputs.search_by_image(fileobj=fio)
    fio.close()
    os.unlink(filename)

  def test_search_input_original_url(self):
    """ search input original url """

    # search by url
    self.app.inputs.search_by_original_url(urls[0])
    self.app.inputs.search_by_original_url(urls[1])

  def test_search_input_metadata(self):
    """ search input meta data """

    # upload image
    image_id = uuid.uuid4().hex
    meta = {'key': 'value'}
    res = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], metadata=meta, allow_duplicate_url=True)

    # simple meta search
    search_res = self.app.inputs.search_by_metadata({"key": "value"})
    # disable this until citus migration fully finishes
    # self.assertGreater(len(search_res), 0)

    search_res = self.app.inputs.search_by_metadata({"key": "value", "key22": "value22"})
    self.assertEqual(len(search_res), 0)

    # nested meta search
    search_res = self.app.inputs.search_by_metadata({"key1": {"key2": {"key3": "value_level34"}}})
    self.assertEqual(len(search_res), 0)

    self.app.inputs.delete(image_id)

  def test_search_input_nested_metadata(self):
    """ search input nested meta data """

    # upload image
    image_id = uuid.uuid4().hex
    meta = {'key': 'value', 'key2': 'value2', 'key3': 'value3'}
    img1 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], metadata=meta, allow_duplicate_url=True)

    # simple meta search
    search_res = self.app.inputs.search_by_metadata({"key": "value", "key2": "value2"})
    # disable this until citus migration fully finishes
    # self.assertGreater(len(search_res), 0)

    # nested meta search
    search_res = self.app.inputs.search_by_metadata({"key1": {"key2": {"key3": "value_level3"}}})
    self.assertEqual(len(search_res), 0)

    # delete first test image
    self.app.inputs.delete(img1.input_id)

    # upload 2nd image
    image_id = uuid.uuid4().hex
    meta = {"key1": {"key2": {"key3": "value_level3", "key3.2": "value3.2"}, "key2.2": "value2.2"}}
    img2 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], metadata=meta, allow_duplicate_url=True)

    # nested meta search
    search_res = self.app.inputs.search_by_metadata({"key1": {"key2": {"key3": "value_level3"}}})
    # disable this until citus migration fully finishes
    # self.assertGreater(len(search_res), 0)

    # delete second test image
    self.app.inputs.delete(img2.input_id)

  def test_search_geo_point(self):
    """ search input geo point data """

    # upload image
    image_id = uuid.uuid4().hex
    geo = Geo(GeoPoint(40.7128, 74.0059))
    img1 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], geo=geo, allow_duplicate_url=True)

    # simple geo search
    search_res = self.app.inputs.search_by_geo(GeoPoint(40.7128, 74.0059))
    # disable this until citus migration fully finishes
    # self.assertGreater(len(search_res), 0)

    search_res = self.app.inputs.search_by_geo(GeoPoint(40.7128, 74.0059), GeoLimit("mile", 10))
    # disable this until citus migration fully finishes
    # self.assertGreater(len(search_res), 0)

    # delete first test image
    self.app.inputs.delete(img1.input_id)

    # upload 2nd image
    image_id = uuid.uuid4().hex
    geo = Geo(GeoPoint(40.7129, 74.0058))
    img2 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], geo=geo, allow_duplicate_url=True)

    search_res = self.app.inputs.search_by_geo(
        GeoPoint(40.7128, 74.0059), GeoLimit("kilometer", 9))
    # disable this until citus migration fully finishes
    # self.assertGreater(len(search_res), 0)

    # delete second test image
    self.app.inputs.delete(img2.input_id)

  def test_search_geo_box(self):
    """ search input geo point data """

    # upload an image
    image_id = uuid.uuid4().hex
    geo = Geo(GeoPoint(40.7129, 74.0058))
    img2 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], geo=geo, allow_duplicate_url=True)

    geo_box = GeoBox(GeoPoint(40.7028, 74.0009), GeoPoint(40.7328, 74.0359))
    search_res = self.app.inputs.search_by_geo(geo_box=geo_box)
    # disable this until citus migration fully finishes
    # self.assertGreater(len(search_res), 0)

    # delete second test image
    self.app.inputs.delete(img2.input_id)

  def test_search_image_url_and_geo(self):
    """ search over image url combined with geo """

    # upload an image
    image_id = uuid.uuid4().hex
    geo = Geo(GeoPoint(40.7129, 74.0058))
    img2 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], geo=geo, allow_duplicate_url=True)

    term1 = clarifai.rest.InputSearchTerm(url=urls[0])
    geo_box = GeoBox(GeoPoint(40.7028, 74.0009), GeoPoint(40.7328, 74.0359))
    term2 = clarifai.rest.InputSearchTerm(geo=Geo(geo_box=geo_box))
    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)
    query.add_term(term2)

    search_res = self.app.inputs.search(query)
    self.assertGreater(len(search_res), 0)
    match = False
    for img in search_res:
      if img.input_id == image_id or img.url == img2.url:
        match = True
        break
    self.assertTrue(match)

    # delete second test image
    self.app.inputs.delete(img2.input_id)

  def test_search_geo_with_input_output_tag(self):
    """ search over input and output tag together with geo info """

    # upload an image
    image_id = uuid.uuid4().hex
    geo = Geo(GeoPoint(40.7129, 74.0058))
    img2 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], geo=geo, concepts=["train"], allow_duplicate_url=True)

    geo_box = GeoBox(GeoPoint(40.7028, 74.0009), GeoPoint(40.7328, 74.0359))
    term1 = clarifai.rest.InputSearchTerm(geo=Geo(geo_box=geo_box))
    term2 = clarifai.rest.InputSearchTerm(concept="train")
    term3 = clarifai.rest.OutputSearchTerm(concept="railway")
    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)
    query.add_term(term2)
    query.add_term(term3)

    search_res = self.app.inputs.search(query, page=1, per_page=20)
    # disable this until citus migration fully finishes
    # self.assertGreater(len(search_res), 0)
    # match = False
    # for img in search_res:
    #  if img.input_id == image_id or img.url == img2.url:
    #    match = True
    #    break
    # self.assertTrue(match)

    # delete second test image
    self.app.inputs.delete(img2.input_id)

  def test_input_query_term(self):
    """ test input query term, same as search_by_annotated_concepts """

    term1 = clarifai.rest.InputSearchTerm(concept='cat')

    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)

    self.app.inputs.search(query)

  def test_output_query_term(self):
    """ test output query term, same as search_by_predicted_concepts """

    term1 = clarifai.rest.OutputSearchTerm(concept='train')

    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)

    self.app.inputs.search(query)

  def test_search_score(self):
    """ test a query, verify the score is available and is descending """

    # upload an image
    image_id = uuid.uuid4().hex
    img1 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[0], concepts=["train"], allow_duplicate_url=True)

    image_id = uuid.uuid4().hex
    img2 = self.app.inputs.create_image_from_url(
        image_id=image_id, url=urls[1], concepts=["train"], allow_duplicate_url=True)

    # search input term and verify the scores are always 1
    term1 = clarifai.rest.InputSearchTerm(concept="train")
    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)

    res = self.app.inputs.search(query)

    # disable this until citus migration fully finishes
    # self.assertGreaterEqual(len(res), 2)
    scores = [one_image.score for one_image in res]
    self.assertEqual(scores, [1] * len(res))

    # search output term and verify the scores are descending
    term1 = clarifai.rest.OutputSearchTerm(concept="train")
    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)

    res = self.app.inputs.search(query)

    # self.assertGreaterEqual(len(res), 2)
    # scores = [one_image.score for one_image in res]
    # self.assertEqual(scores, sorted(scores, reverse=True))

    # clean up
    self.app.inputs.delete(img1.input_id)
    self.app.inputs.delete(img2.input_id)

  def test_search_by_annotated_and_predicted_concepts(self):
    """ more complex query, with both input and output concept search """

    # upload some cat and dog
    img1 = self.app.inputs.create_image_from_url(
        urls[0], concepts=['trains'], allow_duplicate_url=True)

    term1 = clarifai.rest.InputSearchTerm(concept='trains')
    term2 = clarifai.rest.OutputSearchTerm(concept='platform', value=False)
    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)
    query.add_term(term2)

    self.app.inputs.search(query)

    # delete them
    self.app.inputs.delete(img1.input_id)

  def test_complex_combinations(self):
    """ with more complex queries """

    # upload some cat and dog
    img1 = self.app.inputs.create_image_from_url(
        urls[1], concepts=['cat'], allow_duplicate_url=True)
    img2 = self.app.inputs.create_image_from_url(urls[3], allow_duplicate_url=True)

    term1 = clarifai.rest.InputSearchTerm(concept='cat')
    term2 = clarifai.rest.OutputSearchTerm(concept='ceremony', value=True)
    term3 = clarifai.rest.OutputSearchTerm(url=urls[2])

    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)
    query.add_term(term2)
    query.add_term(term3)

    self.app.inputs.search(query)

    # more with all the input search terms
    term1 = clarifai.rest.InputSearchTerm(concept='cat')
    term2 = clarifai.rest.InputSearchTerm(concept='dog', value=False)
    term3 = clarifai.rest.InputSearchTerm(metadata={'name': 'value'})

    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)
    query.add_term(term2)
    query.add_term(term3)

    self.app.inputs.search(query)

    # more with all the output search terms
    term1 = clarifai.rest.OutputSearchTerm(concept='group', value=False)
    term2 = clarifai.rest.OutputSearchTerm(concept='ceremony', value=True)
    term3 = clarifai.rest.OutputSearchTerm(url=urls[1])

    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)
    query.add_term(term2)
    query.add_term(term3)

    self.app.inputs.search(query)

    # more with all the search term combination
    term1 = clarifai.rest.InputSearchTerm(concept='cat')
    term2 = clarifai.rest.InputSearchTerm(concept='dog', value=False)
    term3 = clarifai.rest.InputSearchTerm(metadata={'name': 'value'})
    term4 = clarifai.rest.OutputSearchTerm(concept='group', value=False)
    term5 = clarifai.rest.OutputSearchTerm(concept='ceremony', value=True)
    term6 = clarifai.rest.OutputSearchTerm(url=urls[1])

    query = clarifai.rest.SearchQueryBuilder()
    query.add_term(term1)
    query.add_term(term2)
    query.add_term(term3)
    query.add_term(term4)
    query.add_term(term5)
    query.add_term(term6)

    self.app.inputs.search(query)

    # delete them
    self.app.inputs.delete(img1.input_id)
    self.app.inputs.delete(img2.input_id)


if __name__ == '__main__':
  unittest.main()



import logging
import os
import unittest

from clarifai.rest import ApiError, ClarifaiApp
from clarifai.rest import Image as ClImage
from clarifai.rest import UserError

urls = [
    "https://samples.clarifai.com/metro-north.jpg",
    "https://samples.clarifai.com/wedding.jpg",
    "https://samples.clarifai.com/facebook.png",
    "https://samples.clarifai.com/dog.tiff",
    "https://samples.clarifai.com/penguin.bmp",
]

dir_path = os.path.dirname(os.path.realpath(__file__))
files = [
    os.path.join(os.path.dirname(os.path.abspath(__file__)), "data", "metro-north.jpg"),
    os.path.join(os.path.dirname(os.path.abspath(__file__)), "data", "thai-market.jpg"),
]


class TestV2Tagging(unittest.TestCase):
  """ the unit test for the V2 simplified tagging api
  """

  _multiprocess_can_split_ = True

  @classmethod
  def setUpClass(cls):
    cls.app = ClarifaiApp(log_level=logging.WARN)

  def test_tag_urls(self):
    res = self.app.tag_urls(urls)
    results = res['outputs']
    self.assertEqual(len(results), len(urls))

  def test_tag_urls_with_model_id(self):
    # general model
    res = self.app.tag_urls(urls, model_id='aaa03c23b3724a16a56b629203edc62c')
    results = res['outputs']
    self.assertEqual(len(results), len(urls))

    # travel model
    res = self.app.tag_urls(urls, model_id='eee28c313d69466f836ab83287a54ed9')
    results = res['outputs']
    self.assertEqual(len(results), len(urls))

  def test_tag_non_urls(self):
    """ tag non list or non urls """

    image_url = 'https://samples.clarifai.com/toddler-flowers.jpeg'
    urls = image_url

    with self.assertRaises(UserError) as ue:
      self.app.tag_urls(urls)

  def test_tag_overloaded_urls(self):
    """ tag more than 128 urls """

    image_url = 'https://samples.clarifai.com/toddler-flowers.jpeg'
    urls = [image_url] * 129

    with self.assertRaises(UserError) as ue:
      self.app.tag_urls(urls)

    urls = [image_url] * 16
    try:
      res = self.app.tag_urls(urls)
    except ApiError as e:
      # ignore the partial errors
      if e.response.status_code == 207:
        pass
      else:
        raise e

    self.assertEqual(len(res['outputs']), len(urls))

  def test_tag_files(self):
    res = self.app.tag_files(files)
    results = res['outputs']
    self.assertEqual(len(results), len(files))

  def test_tag_non_files(self):
    """ tag non list or non filename """

    one_file = files[0]
    fs = one_file

    with self.assertRaises(UserError):
      self.app.tag_files(fs)

  def test_tag_overloaded_files(self):
    """ tag more than 128 files """

    file_one = files[0]
    fs = [file_one] * 129

    with self.assertRaises(UserError):
      self.app.tag_files(fs)

    fs = [file_one] * 16
    self.app.tag_files(fs)

  def test_shortcut_predict_concept_function(self):
    img1 = ClImage(url=urls[0])
    res = self.app.api.predict_concepts([img1])
    self.assertEqual(10000, res['status']['code'])

  def test_shortcut_predict_colors_function(self):
    img1 = ClImage(url=urls[0])
    res = self.app.api.predict_colors([img1])
    self.assertEqual(10000, res['status']['code'])

  def test_shortcut_predict_embed_function(self):
    img1 = ClImage(url=urls[0])
    res = self.app.api.predict_embed([img1])
    self.assertEqual(10000, res['status']['code'])


if __name__ == '__main__':
  unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/clarifai/api/vocab.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from clarifai.rest.grpc.proto.clarifai.api import common_pb2 as proto_dot_clarifai_dot_api_dot_common__pb2
from clarifai.rest.grpc.proto.clarifai.api import concept_pb2 as proto_dot_clarifai_dot_api_dot_concept__pb2
from clarifai.rest.grpc.proto.clarifai.api.status import status_pb2 as proto_dot_clarifai_dot_api_dot_status_dot_status__pb2
from clarifai.rest.grpc.proto.clarifai.api.utils import extensions_pb2 as proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/clarifai/api/vocab.proto',
  package='clarifai.api',
  syntax='proto3',
  serialized_pb=_b('\n\x1eproto/clarifai/api/vocab.proto\x12\x0c\x63larifai.api\x1a\x1fproto/clarifai/api/common.proto\x1a proto/clarifai/api/concept.proto\x1a&proto/clarifai/api/status/status.proto\x1a)proto/clarifai/api/utils/extensions.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"v\n\x05Vocab\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x04 \x01(\t\x12.\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"T\n\x0fGetVocabRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\x10\n\x08vocab_id\x18\x02 \x01(\t\"d\n\x11ListVocabsRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\x0c\n\x04page\x18\x02 \x01(\r\x12\x10\n\x08per_page\x18\x03 \x01(\r\"i\n\x11PostVocabsRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12#\n\x06vocabs\x18\x02 \x03(\x0b\x32\x13.clarifai.api.Vocab\"z\n\x12PatchVocabsRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12#\n\x06vocabs\x18\x02 \x03(\x0b\x32\x13.clarifai.api.Vocab\x12\x0e\n\x06\x61\x63tion\x18\x03 \x01(\t\"W\n\x12\x44\x65leteVocabRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\x10\n\x08vocab_id\x18\x02 \x01(\t\"g\n\x13\x44\x65leteVocabsRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\x0b\n\x03ids\x18\x02 \x03(\t\x12\x12\n\ndelete_all\x18\x03 \x01(\x08\"}\n\x18ListVocabConceptsRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\x10\n\x08vocab_id\x18\x02 \x01(\t\x12\x0c\n\x04page\x18\x03 \x01(\r\x12\x10\n\x08per_page\x18\x04 \x01(\r\"\x86\x01\n\x18PostVocabConceptsRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\x10\n\x08vocab_id\x18\x02 \x01(\t\x12\'\n\x08\x63oncepts\x18\x03 \x03(\x0b\x32\x15.clarifai.api.Concept\"r\n\x19\x44\x65leteVocabConceptRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\x10\n\x08vocab_id\x18\x02 \x01(\t\x12\x12\n\nconcept_id\x18\x03 \x01(\t\"\x80\x01\n\x1a\x44\x65leteVocabConceptsRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\x10\n\x08vocab_id\x18\x02 \x01(\t\x12\x0b\n\x03ids\x18\x03 \x03(\t\x12\x12\n\ndelete_all\x18\x04 \x01(\x08\"f\n\x13SingleVocabResponse\x12+\n\x06status\x18\x01 \x01(\x0b\x32\x1b.clarifai.api.status.Status\x12\"\n\x05vocab\x18\x02 \x01(\x0b\x32\x13.clarifai.api.Vocab\"l\n\x12MultiVocabResponse\x12+\n\x06status\x18\x01 \x01(\x0b\x32\x1b.clarifai.api.status.Status\x12)\n\x06vocabs\x18\x02 \x03(\x0b\x32\x13.clarifai.api.VocabB\x04\x80\xb5\x18\x01\x42$Z\x03\x61pi\xa2\x02\x04\x43\x41IP\xc2\x02\x01_\xca\x02\x11\x43larifai\\Internalb\x06proto3')
  ,
  dependencies=[proto_dot_clarifai_dot_api_dot_common__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_concept__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_status_dot_status__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_VOCAB = _descriptor.Descriptor(
  name='Vocab',
  full_name='clarifai.api.Vocab',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='clarifai.api.Vocab.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='clarifai.api.Vocab.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='clarifai.api.Vocab.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='app_id', full_name='clarifai.api.Vocab.app_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='clarifai.api.Vocab.created_at', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=231,
  serialized_end=349,
)


_GETVOCABREQUEST = _descriptor.Descriptor(
  name='GetVocabRequest',
  full_name='clarifai.api.GetVocabRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.GetVocabRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocab_id', full_name='clarifai.api.GetVocabRequest.vocab_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=351,
  serialized_end=435,
)


_LISTVOCABSREQUEST = _descriptor.Descriptor(
  name='ListVocabsRequest',
  full_name='clarifai.api.ListVocabsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.ListVocabsRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='page', full_name='clarifai.api.ListVocabsRequest.page', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='per_page', full_name='clarifai.api.ListVocabsRequest.per_page', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=437,
  serialized_end=537,
)


_POSTVOCABSREQUEST = _descriptor.Descriptor(
  name='PostVocabsRequest',
  full_name='clarifai.api.PostVocabsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.PostVocabsRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocabs', full_name='clarifai.api.PostVocabsRequest.vocabs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=539,
  serialized_end=644,
)


_PATCHVOCABSREQUEST = _descriptor.Descriptor(
  name='PatchVocabsRequest',
  full_name='clarifai.api.PatchVocabsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.PatchVocabsRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocabs', full_name='clarifai.api.PatchVocabsRequest.vocabs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='action', full_name='clarifai.api.PatchVocabsRequest.action', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=646,
  serialized_end=768,
)


_DELETEVOCABREQUEST = _descriptor.Descriptor(
  name='DeleteVocabRequest',
  full_name='clarifai.api.DeleteVocabRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.DeleteVocabRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocab_id', full_name='clarifai.api.DeleteVocabRequest.vocab_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=770,
  serialized_end=857,
)


_DELETEVOCABSREQUEST = _descriptor.Descriptor(
  name='DeleteVocabsRequest',
  full_name='clarifai.api.DeleteVocabsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.DeleteVocabsRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ids', full_name='clarifai.api.DeleteVocabsRequest.ids', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delete_all', full_name='clarifai.api.DeleteVocabsRequest.delete_all', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=859,
  serialized_end=962,
)


_LISTVOCABCONCEPTSREQUEST = _descriptor.Descriptor(
  name='ListVocabConceptsRequest',
  full_name='clarifai.api.ListVocabConceptsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.ListVocabConceptsRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocab_id', full_name='clarifai.api.ListVocabConceptsRequest.vocab_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='page', full_name='clarifai.api.ListVocabConceptsRequest.page', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='per_page', full_name='clarifai.api.ListVocabConceptsRequest.per_page', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=964,
  serialized_end=1089,
)


_POSTVOCABCONCEPTSREQUEST = _descriptor.Descriptor(
  name='PostVocabConceptsRequest',
  full_name='clarifai.api.PostVocabConceptsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.PostVocabConceptsRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocab_id', full_name='clarifai.api.PostVocabConceptsRequest.vocab_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='concepts', full_name='clarifai.api.PostVocabConceptsRequest.concepts', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1092,
  serialized_end=1226,
)


_DELETEVOCABCONCEPTREQUEST = _descriptor.Descriptor(
  name='DeleteVocabConceptRequest',
  full_name='clarifai.api.DeleteVocabConceptRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.DeleteVocabConceptRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocab_id', full_name='clarifai.api.DeleteVocabConceptRequest.vocab_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='concept_id', full_name='clarifai.api.DeleteVocabConceptRequest.concept_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1228,
  serialized_end=1342,
)


_DELETEVOCABCONCEPTSREQUEST = _descriptor.Descriptor(
  name='DeleteVocabConceptsRequest',
  full_name='clarifai.api.DeleteVocabConceptsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.DeleteVocabConceptsRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocab_id', full_name='clarifai.api.DeleteVocabConceptsRequest.vocab_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ids', full_name='clarifai.api.DeleteVocabConceptsRequest.ids', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delete_all', full_name='clarifai.api.DeleteVocabConceptsRequest.delete_all', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1345,
  serialized_end=1473,
)


_SINGLEVOCABRESPONSE = _descriptor.Descriptor(
  name='SingleVocabResponse',
  full_name='clarifai.api.SingleVocabResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='clarifai.api.SingleVocabResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocab', full_name='clarifai.api.SingleVocabResponse.vocab', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1475,
  serialized_end=1577,
)


_MULTIVOCABRESPONSE = _descriptor.Descriptor(
  name='MultiVocabResponse',
  full_name='clarifai.api.MultiVocabResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='clarifai.api.MultiVocabResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vocabs', full_name='clarifai.api.MultiVocabResponse.vocabs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1579,
  serialized_end=1687,
)

_VOCAB.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_GETVOCABREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_LISTVOCABSREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_POSTVOCABSREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_POSTVOCABSREQUEST.fields_by_name['vocabs'].message_type = _VOCAB
_PATCHVOCABSREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_PATCHVOCABSREQUEST.fields_by_name['vocabs'].message_type = _VOCAB
_DELETEVOCABREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_DELETEVOCABSREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_LISTVOCABCONCEPTSREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_POSTVOCABCONCEPTSREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_POSTVOCABCONCEPTSREQUEST.fields_by_name['concepts'].message_type = proto_dot_clarifai_dot_api_dot_concept__pb2._CONCEPT
_DELETEVOCABCONCEPTREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_DELETEVOCABCONCEPTSREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_SINGLEVOCABRESPONSE.fields_by_name['status'].message_type = proto_dot_clarifai_dot_api_dot_status_dot_status__pb2._STATUS
_SINGLEVOCABRESPONSE.fields_by_name['vocab'].message_type = _VOCAB
_MULTIVOCABRESPONSE.fields_by_name['status'].message_type = proto_dot_clarifai_dot_api_dot_status_dot_status__pb2._STATUS
_MULTIVOCABRESPONSE.fields_by_name['vocabs'].message_type = _VOCAB
DESCRIPTOR.message_types_by_name['Vocab'] = _VOCAB
DESCRIPTOR.message_types_by_name['GetVocabRequest'] = _GETVOCABREQUEST
DESCRIPTOR.message_types_by_name['ListVocabsRequest'] = _LISTVOCABSREQUEST
DESCRIPTOR.message_types_by_name['PostVocabsRequest'] = _POSTVOCABSREQUEST
DESCRIPTOR.message_types_by_name['PatchVocabsRequest'] = _PATCHVOCABSREQUEST
DESCRIPTOR.message_types_by_name['DeleteVocabRequest'] = _DELETEVOCABREQUEST
DESCRIPTOR.message_types_by_name['DeleteVocabsRequest'] = _DELETEVOCABSREQUEST
DESCRIPTOR.message_types_by_name['ListVocabConceptsRequest'] = _LISTVOCABCONCEPTSREQUEST
DESCRIPTOR.message_types_by_name['PostVocabConceptsRequest'] = _POSTVOCABCONCEPTSREQUEST
DESCRIPTOR.message_types_by_name['DeleteVocabConceptRequest'] = _DELETEVOCABCONCEPTREQUEST
DESCRIPTOR.message_types_by_name['DeleteVocabConceptsRequest'] = _DELETEVOCABCONCEPTSREQUEST
DESCRIPTOR.message_types_by_name['SingleVocabResponse'] = _SINGLEVOCABRESPONSE
DESCRIPTOR.message_types_by_name['MultiVocabResponse'] = _MULTIVOCABRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Vocab = _reflection.GeneratedProtocolMessageType('Vocab', (_message.Message,), dict(
  DESCRIPTOR = _VOCAB,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.Vocab)
  ))
_sym_db.RegisterMessage(Vocab)

GetVocabRequest = _reflection.GeneratedProtocolMessageType('GetVocabRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETVOCABREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.GetVocabRequest)
  ))
_sym_db.RegisterMessage(GetVocabRequest)

ListVocabsRequest = _reflection.GeneratedProtocolMessageType('ListVocabsRequest', (_message.Message,), dict(
  DESCRIPTOR = _LISTVOCABSREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.ListVocabsRequest)
  ))
_sym_db.RegisterMessage(ListVocabsRequest)

PostVocabsRequest = _reflection.GeneratedProtocolMessageType('PostVocabsRequest', (_message.Message,), dict(
  DESCRIPTOR = _POSTVOCABSREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.PostVocabsRequest)
  ))
_sym_db.RegisterMessage(PostVocabsRequest)

PatchVocabsRequest = _reflection.GeneratedProtocolMessageType('PatchVocabsRequest', (_message.Message,), dict(
  DESCRIPTOR = _PATCHVOCABSREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.PatchVocabsRequest)
  ))
_sym_db.RegisterMessage(PatchVocabsRequest)

DeleteVocabRequest = _reflection.GeneratedProtocolMessageType('DeleteVocabRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETEVOCABREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.DeleteVocabRequest)
  ))
_sym_db.RegisterMessage(DeleteVocabRequest)

DeleteVocabsRequest = _reflection.GeneratedProtocolMessageType('DeleteVocabsRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETEVOCABSREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.DeleteVocabsRequest)
  ))
_sym_db.RegisterMessage(DeleteVocabsRequest)

ListVocabConceptsRequest = _reflection.GeneratedProtocolMessageType('ListVocabConceptsRequest', (_message.Message,), dict(
  DESCRIPTOR = _LISTVOCABCONCEPTSREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.ListVocabConceptsRequest)
  ))
_sym_db.RegisterMessage(ListVocabConceptsRequest)

PostVocabConceptsRequest = _reflection.GeneratedProtocolMessageType('PostVocabConceptsRequest', (_message.Message,), dict(
  DESCRIPTOR = _POSTVOCABCONCEPTSREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.PostVocabConceptsRequest)
  ))
_sym_db.RegisterMessage(PostVocabConceptsRequest)

DeleteVocabConceptRequest = _reflection.GeneratedProtocolMessageType('DeleteVocabConceptRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETEVOCABCONCEPTREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.DeleteVocabConceptRequest)
  ))
_sym_db.RegisterMessage(DeleteVocabConceptRequest)

DeleteVocabConceptsRequest = _reflection.GeneratedProtocolMessageType('DeleteVocabConceptsRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETEVOCABCONCEPTSREQUEST,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.DeleteVocabConceptsRequest)
  ))
_sym_db.RegisterMessage(DeleteVocabConceptsRequest)

SingleVocabResponse = _reflection.GeneratedProtocolMessageType('SingleVocabResponse', (_message.Message,), dict(
  DESCRIPTOR = _SINGLEVOCABRESPONSE,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.SingleVocabResponse)
  ))
_sym_db.RegisterMessage(SingleVocabResponse)

MultiVocabResponse = _reflection.GeneratedProtocolMessageType('MultiVocabResponse', (_message.Message,), dict(
  DESCRIPTOR = _MULTIVOCABRESPONSE,
  __module__ = 'proto.clarifai.api.vocab_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.MultiVocabResponse)
  ))
_sym_db.RegisterMessage(MultiVocabResponse)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z\003api\242\002\004CAIP\302\002\001_\312\002\021Clarifai\\Internal'))
_MULTIVOCABRESPONSE.fields_by_name['vocabs'].has_options = True
_MULTIVOCABRESPONSE.fields_by_name['vocabs']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/clarifai/api/output.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from clarifai.rest.grpc.proto.clarifai.api import common_pb2 as proto_dot_clarifai_dot_api_dot_common__pb2
from clarifai.rest.grpc.proto.clarifai.api import data_pb2 as proto_dot_clarifai_dot_api_dot_data__pb2
from clarifai.rest.grpc.proto.clarifai.api import input_pb2 as proto_dot_clarifai_dot_api_dot_input__pb2
from clarifai.rest.grpc.proto.clarifai.api import model_pb2 as proto_dot_clarifai_dot_api_dot_model__pb2
from clarifai.rest.grpc.proto.clarifai.api.status import status_pb2 as proto_dot_clarifai_dot_api_dot_status_dot_status__pb2
from clarifai.rest.grpc.proto.clarifai.api.utils import extensions_pb2 as proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/clarifai/api/output.proto',
  package='clarifai.api',
  syntax='proto3',
  serialized_pb=_b('\n\x1fproto/clarifai/api/output.proto\x12\x0c\x63larifai.api\x1a\x1fproto/clarifai/api/common.proto\x1a\x1dproto/clarifai/api/data.proto\x1a\x1eproto/clarifai/api/input.proto\x1a\x1eproto/clarifai/api/model.proto\x1a&proto/clarifai/api/status/status.proto\x1a)proto/clarifai/api/utils/extensions.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xdb\x01\n\x06Output\x12\n\n\x02id\x18\x01 \x01(\t\x12+\n\x06status\x18\x02 \x01(\x0b\x32\x1b.clarifai.api.status.Status\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\"\n\x05model\x18\x04 \x01(\x0b\x32\x13.clarifai.api.Model\x12\"\n\x05input\x18\x05 \x01(\x0b\x32\x13.clarifai.api.Input\x12 \n\x04\x64\x61ta\x18\x06 \x01(\x0b\x32\x12.clarifai.api.Data\"o\n\x13MultiOutputResponse\x12+\n\x06status\x18\x01 \x01(\x0b\x32\x1b.clarifai.api.status.Status\x12+\n\x07outputs\x18\x02 \x03(\x0b\x32\x14.clarifai.api.OutputB\x04\x80\xb5\x18\x01\x42$Z\x03\x61pi\xa2\x02\x04\x43\x41IP\xc2\x02\x01_\xca\x02\x11\x43larifai\\Internalb\x06proto3')
  ,
  dependencies=[proto_dot_clarifai_dot_api_dot_common__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_data__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_input__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_model__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_status_dot_status__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_OUTPUT = _descriptor.Descriptor(
  name='Output',
  full_name='clarifai.api.Output',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='clarifai.api.Output.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='clarifai.api.Output.status', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='clarifai.api.Output.created_at', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='model', full_name='clarifai.api.Output.model', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input', full_name='clarifai.api.Output.input', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='clarifai.api.Output.data', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=294,
  serialized_end=513,
)


_MULTIOUTPUTRESPONSE = _descriptor.Descriptor(
  name='MultiOutputResponse',
  full_name='clarifai.api.MultiOutputResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='clarifai.api.MultiOutputResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outputs', full_name='clarifai.api.MultiOutputResponse.outputs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=515,
  serialized_end=626,
)

_OUTPUT.fields_by_name['status'].message_type = proto_dot_clarifai_dot_api_dot_status_dot_status__pb2._STATUS
_OUTPUT.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_OUTPUT.fields_by_name['model'].message_type = proto_dot_clarifai_dot_api_dot_model__pb2._MODEL
_OUTPUT.fields_by_name['input'].message_type = proto_dot_clarifai_dot_api_dot_input__pb2._INPUT
_OUTPUT.fields_by_name['data'].message_type = proto_dot_clarifai_dot_api_dot_data__pb2._DATA
_MULTIOUTPUTRESPONSE.fields_by_name['status'].message_type = proto_dot_clarifai_dot_api_dot_status_dot_status__pb2._STATUS
_MULTIOUTPUTRESPONSE.fields_by_name['outputs'].message_type = _OUTPUT
DESCRIPTOR.message_types_by_name['Output'] = _OUTPUT
DESCRIPTOR.message_types_by_name['MultiOutputResponse'] = _MULTIOUTPUTRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Output = _reflection.GeneratedProtocolMessageType('Output', (_message.Message,), dict(
  DESCRIPTOR = _OUTPUT,
  __module__ = 'proto.clarifai.api.output_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.Output)
  ))
_sym_db.RegisterMessage(Output)

MultiOutputResponse = _reflection.GeneratedProtocolMessageType('MultiOutputResponse', (_message.Message,), dict(
  DESCRIPTOR = _MULTIOUTPUTRESPONSE,
  __module__ = 'proto.clarifai.api.output_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.MultiOutputResponse)
  ))
_sym_db.RegisterMessage(MultiOutputResponse)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z\003api\242\002\004CAIP\302\002\001_\312\002\021Clarifai\\Internal'))
_MULTIOUTPUTRESPONSE.fields_by_name['outputs'].has_options = True
_MULTIOUTPUTRESPONSE.fields_by_name['outputs']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001'))
# @@protoc_insertion_point(module_scope)



import unittest

from parameterized import parameterized

from clarifai.rest.grpc.grpc_json_channel import _pick_proper_endpoint

MODEL_VERSION_RESOURCES = [
    ('https://url.com/{model_id}/outputs', 'POST'),
    ('https://url.com/{model_id}/versions/{version_id}/outputs', 'POST'),
    ('https://url.com/{user_app_id.user_id}/apps/{user_app_id.app_id}/models/{model_id}/outputs',
     'POST'),
]

REVERSED_MODEL_VERSION_RESOURCES = [
    ('https://url.com/{model_id}/versions/{version_id}/outputs', 'POST'),
    ('https://url.com/{user_app_id.user_id}/apps/{user_app_id.app_id}/models/{model_id}/outputs',
     'POST'),
    ('https://url.com/{model_id}/outputs', 'POST'),
]

NO_ARGUMENT_RESOURCES = [('https://url.com/workflows', 'GET')]


class TestPickProperEndpoint(unittest.TestCase):
  _multiprocess_can_split_ = True

  @parameterized.expand([(MODEL_VERSION_RESOURCES,), (REVERSED_MODEL_VERSION_RESOURCES,)])
  def test_should_pick_correct_url_when_model_id(self, resources):
    request_dict = {
        'model_id': 'SOME-MODEL-ID',
    }
    picked_url, _ = _pick_proper_endpoint(resources, request_dict)
    self.assertEqual('https://url.com/SOME-MODEL-ID/outputs', picked_url)

  @parameterized.expand([(MODEL_VERSION_RESOURCES,), (REVERSED_MODEL_VERSION_RESOURCES,)])
  def test_should_pick_correct_url_when_model_id_and_version_id(self, resources):
    request_dict = {
        'model_id': 'SOME-MODEL-ID',
        'version_id': 'SOME-VERSION-ID',
    }
    picked_url, _ = _pick_proper_endpoint(resources, request_dict)
    self.assertEqual('https://url.com/SOME-MODEL-ID/versions/SOME-VERSION-ID/outputs', picked_url)

  @parameterized.expand([(MODEL_VERSION_RESOURCES,), (REVERSED_MODEL_VERSION_RESOURCES,)])
  def test_should_pick_correct_url_when_user_and_model_id(self, resources):
    request_dict = {
        'user_id': 'USER-ID',
        'app_id': 'APP-ID',
        'model_id': 'SOME-MODEL-ID',
    }
    picked_url, _ = _pick_proper_endpoint(resources, request_dict)
    self.assertEqual('https://url.com/USER-ID/apps/APP-ID/models/SOME-MODEL-ID/outputs',
                     picked_url)

  def test_should_pick_the_url_when_no_argument(self):
    request_dict = {}
    picked_url, _ = _pick_proper_endpoint(NO_ARGUMENT_RESOURCES, request_dict)
    self.assertEqual('https://url.com/workflows', picked_url)



import unittest

from clarifai.rest import ApiStatus


class TestStatus(unittest.TestCase):
  """
  unit test for api status
  """

  _multiprocess_can_split_ = True

  def test_api_status(self):
    # make the status
    item = {'code': 10000, 'description': 'Ok'}

    status = ApiStatus(item)
    data = status.dict()
    self.assertEqual(10000, data['status']['code'])
    self.assertEqual('Ok', data['status']['description'])



import json
import logging
import os
import re
import time
import unittest

import responses

from clarifai.rest import ApiClient

urls = [
    "https://samples.clarifai.com/metro-north.jpg",
    "https://samples.clarifai.com/wedding.jpg",
    "https://samples.clarifai.com/facebook.png",
    "https://samples.clarifai.com/dog.tiff",
    "https://samples.clarifai.com/penguin.bmp",
]


class TestApiClient(unittest.TestCase):
  """ the unit test for the ApiClient class

      This test is only to test the RESTful api endpoint calls
  """

  _multiprocess_can_split_ = True

  @classmethod
  def setUpClass(cls):
    cls.api = ApiClient(log_level=logging.WARN)

  def test_get_token(self):
    # Make sure by default the client gets a token so it can handle requests.
    with self.assertRaises(DeprecationWarning):
      self.api.get_token()

  def test_set_token(self):
    with self.assertRaises(DeprecationWarning):
      self.api.set_token('some-token')

  def test_api_init(self):
    api = ApiClient()

    if os.environ.get('CLARIFAI_API_KEY'):
      key = os.environ['CLARIFAI_API_KEY']

      api = ApiClient(api_key=key)

      if os.environ.get('CLARIFAI_API_BASE'):
        api_base = os.environ['CLARIFAI_API_BASE']
        api = ApiClient(api_key=key, base_url=api_base)

  def test_delete_token(self):
    with self.assertRaises(DeprecationWarning):
      self.api.delete_token()


class TestApiExceptions(unittest.TestCase):
  """ test api errors """

  @classmethod
  def setUpClass(cls):
    cls.api = ApiClient()

  def test_bad_gateway_error(self):
    """ test bad gateway error
    """

    mocked_obj = {u'status': {u'code': 10020, u'description': u'Bad gateway'}}
    mocked_json = json.dumps(mocked_obj)

    mocked_good_json = '''{
  "status": {
    "code": 10000,
    "description": "Ok"
  },
  "concepts": [
    {
      "id": "lambdsfsdfo773337",
      "name": "lamborghini",
      "created_at": "2016-09-28T21:15:19Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    },
    {
      "id": "lambo7werswdfw77",
      "name": "lamborghini",
      "created_at": "2016-09-28T21:15:19Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    },
    {
      "id": "lambo777",
      "name": "lamborghini",
      "created_at": "2016-09-28T21:14:49Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    },
    {
      "id": "mattid67",
      "name": "mattid67",
      "created_at": "2016-09-28T20:46:17Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    },
    {
      "id": "ferrari",
      "name": "ferrari",
      "created_at": "2016-09-28T20:45:59Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    }
  ]
}'''

    status_codes = [500, 502, 503]

    for status_code in status_codes:
      with responses.RequestsMock() as rsps:
        url_re = re.compile(r'.*clarifai.com/v2/concepts.*')
        rsps.add(
            responses.GET,
            url_re,
            body=mocked_json,
            status=status_code,
            content_type='application/json')
        rsps.add(
            responses.GET,
            url_re,
            body=mocked_json,
            status=status_code,
            content_type='application/json')
        rsps.add(
            responses.GET,
            url_re,
            body=mocked_good_json,
            status=200,
            content_type='application/json')

        res = self.api.get_concepts(page=1, per_page=5)

        self.assertDictContainsSubset({'status': {'code': 10000, 'description': 'Ok'}}, res)
        self.assertEqual(len(res['concepts']), 5)

  def test_429_throttling(self):
    """
    test that the client sleeps an appropriate length of time and retries on 429 throttling error
    """

    mocked_error_json = '''{
  "status": {
    "code": 11005,
    "description": "Too Many Requests",
    "details": "Making too many requests, error_details: exceeded limit of 10 requests per second"
  }
}'''

    mocked_success_response = '''{
  "status": {
    "code": 10000,
    "description": "Ok"
  },
  "concepts": [
    {
      "id": "lambdsfsdfo773337",
      "name": "lamborghini",
      "created_at": "2016-09-28T21:15:19Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    },
    {
      "id": "lambo7werswdfw77",
      "name": "lamborghini",
      "created_at": "2016-09-28T21:15:19Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    },
    {
      "id": "lambo777",
      "name": "lamborghini",
      "created_at": "2016-09-28T21:14:49Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    },
    {
      "id": "mattid67",
      "name": "mattid67",
      "created_at": "2016-09-28T20:46:17Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    },
    {
      "id": "ferrari",
      "name": "ferrari",
      "created_at": "2016-09-28T20:45:59Z",
      "app_id": "ib81c84d5b2341858b86da18a2bd21d2"
    }
  ]
}'''

    with responses.RequestsMock() as rsps:
      url_re = re.compile(r'.*clarifai.com/v2/concepts.*')
      rsps.add(
          responses.GET,
          url_re,
          body=mocked_error_json,
          status=429,
          content_type='application/json')
      rsps.add(
          responses.GET,
          url_re,
          body=mocked_success_response,
          status=200,
          content_type='application/json')

      ts_start = time.time()
      res = self.api.get_concepts(page=1, per_page=5)
      ts_elapsed = time.time() - ts_start

    self.assertDictContainsSubset({'status': {'code': 10000, 'description': 'Ok'}}, res)
    self.assertEqual(len(res['concepts']), 5)
    self.assertGreaterEqual(ts_elapsed, 1.000)

    # another test, with two throttles
    with responses.RequestsMock() as rsps:
      url_re = re.compile(r'.*clarifai.com/v2/concepts.*')
      rsps.add(
          responses.GET,
          url_re,
          body=mocked_error_json,
          status=429,
          content_type='application/json')
      rsps.add(
          responses.GET,
          url_re,
          body=mocked_error_json,
          status=429,
          content_type='application/json')
      rsps.add(
          responses.GET,
          url_re,
          body=mocked_success_response,
          status=200,
          content_type='application/json')

      ts_start = time.time()
      res = self.api.get_concepts(page=1, per_page=5)
      ts_elapsed = time.time() - ts_start

      self.assertDictContainsSubset({'status': {'code': 10000, 'description': 'Ok'}}, res)
      self.assertEqual(len(res['concepts']), 5)
      self.assertGreaterEqual(ts_elapsed, 3.000)


if __name__ == '__main__':
  unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/clarifai/api/utils/extensions.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/clarifai/api/utils/extensions.proto',
  package='clarifai.api.utils',
  syntax='proto3',
  serialized_pb=_b('\n)proto/clarifai/api/utils/extensions.proto\x12\x12\x63larifai.api.utils\x1a google/protobuf/descriptor.proto:9\n\x10\x63l_show_if_empty\x12\x1d.google.protobuf.FieldOptions\x18\xd0\x86\x03 \x01(\x08:4\n\x0b\x63l_moretags\x12\x1d.google.protobuf.FieldOptions\x18\xd1\x86\x03 \x01(\t:9\n\x10\x63l_default_float\x12\x1d.google.protobuf.FieldOptions\x18\xda\x86\x03 \x01(\x02\x42,Z\x05utils\xa2\x02\x04\x43\x41IP\xc2\x02\x01_\xca\x02\x17\x43larifai\\Internal\\Utilsb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,])


CL_SHOW_IF_EMPTY_FIELD_NUMBER = 50000
cl_show_if_empty = _descriptor.FieldDescriptor(
  name='cl_show_if_empty', full_name='clarifai.api.utils.cl_show_if_empty', index=0,
  number=50000, type=8, cpp_type=7, label=1,
  has_default_value=False, default_value=False,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  options=None, file=DESCRIPTOR)
CL_MORETAGS_FIELD_NUMBER = 50001
cl_moretags = _descriptor.FieldDescriptor(
  name='cl_moretags', full_name='clarifai.api.utils.cl_moretags', index=1,
  number=50001, type=9, cpp_type=9, label=1,
  has_default_value=False, default_value=_b("").decode('utf-8'),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  options=None, file=DESCRIPTOR)
CL_DEFAULT_FLOAT_FIELD_NUMBER = 50010
cl_default_float = _descriptor.FieldDescriptor(
  name='cl_default_float', full_name='clarifai.api.utils.cl_default_float', index=2,
  number=50010, type=2, cpp_type=6, label=1,
  has_default_value=False, default_value=float(0),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  options=None, file=DESCRIPTOR)

DESCRIPTOR.extensions_by_name['cl_show_if_empty'] = cl_show_if_empty
DESCRIPTOR.extensions_by_name['cl_moretags'] = cl_moretags
DESCRIPTOR.extensions_by_name['cl_default_float'] = cl_default_float
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(cl_show_if_empty)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(cl_moretags)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(cl_default_float)

DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z\005utils\242\002\004CAIP\302\002\001_\312\002\027Clarifai\\Internal\\Utils'))
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

import logging
import unittest

from clarifai.rest import ClarifaiApp

IMAGE_URL = "https://samples.clarifai.com/metro-north.jpg"


class TestPublicModels(unittest.TestCase):
  _multiprocess_can_split_ = True
  to_cleanup = []

  @classmethod
  def setUpClass(cls):
    cls.app = ClarifaiApp(log_level=logging.WARN)

  def test_predict_with_apparel_model(self):
    res = self.app.public_models.apparel_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_celebrity_model(self):
    res = self.app.public_models.celebrity_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_color_mode(self):
    res = self.app.public_models.color_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_demographics_mode(self):
    res = self.app.public_models.demographics_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_face_detection_model(self):
    res = self.app.public_models.face_detection_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_face_embedding_model(self):
    res = self.app.public_models.face_embedding_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_focus_model(self):
    res = self.app.public_models.focus_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_food_model(self):
    res = self.app.public_models.food_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_general_embedding_model(self):
    res = self.app.public_models.general_embedding_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_general_model(self):
    res = self.app.public_models.general_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_landscape_quality_model(self):
    res = self.app.public_models.landscape_quality_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_logo_model(self):
    res = self.app.public_models.logo_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_moderation_model(self):
    res = self.app.public_models.moderation_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_nsfw_model(self):
    res = self.app.public_models.nsfw_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_portrait_model(self):
    res = self.app.public_models.portrait_quality_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_textures_and_patterns_model(self):
    res = self.app.public_models.textures_and_patterns_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_travel_model(self):
    res = self.app.public_models.travel_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_wedding_model(self):
    res = self.app.public_models.wedding_model.predict_by_url(url=IMAGE_URL)
    self.assertEqual(10000, res['status']['code'])

  def test_predict_with_segmentation_model(self):
    # only runs this test in prod as the segmentation models only exist in prod
    if self.app.api.base_url == 'api.clarifai.com':
      # test people segmentation model
      m = self.app.models.get(model_id='9bef4f02cb40e4709fa254d597b2942d')
      m.model_version = 'db92abe2e6714d7992b359c1fc269ce5'
      res = m.predict_by_url('https://samples.clarifai.com/metro-north.jpg')

      # ensure that only 'person' concepts are returned
      # this tests for the correct filtering of concepts in the backend
      for region in res['outputs'][0]['data']['regions']:
        for concept in region['data']['concepts']:
          self.assertEqual('person', concept['name'])

      # test car segmentation
      m = self.app.models.get(model_id='b17ab715bba448228ef916caaaf8d35b')
      m.model_version = 'fc92eca9d7cb45cc8832ab40ddc95c3c'
      res = m.predict_by_url('https://samples.clarifai.com/metro-north.jpg')

      # ensure that only 'car' concepts are returned
      for region in res['outputs'][0]['data']['regions']:
        for concept in region['data']['concepts']:
          self.assertEqual('car', concept['name'])


if __name__ == '__main__':
  unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/clarifai/api/search.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from clarifai.rest.grpc.proto.clarifai.api import common_pb2 as proto_dot_clarifai_dot_api_dot_common__pb2
from clarifai.rest.grpc.proto.clarifai.api import input_pb2 as proto_dot_clarifai_dot_api_dot_input__pb2
from clarifai.rest.grpc.proto.clarifai.api import output_pb2 as proto_dot_clarifai_dot_api_dot_output__pb2
from clarifai.rest.grpc.proto.clarifai.api.status import status_pb2 as proto_dot_clarifai_dot_api_dot_status_dot_status__pb2
from clarifai.rest.grpc.proto.clarifai.api.utils import extensions_pb2 as proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2
from clarifai.rest.grpc.proto.clarifai.utils.pagination import pagination_pb2 as proto_dot_clarifai_dot_utils_dot_pagination_dot_pagination__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/clarifai/api/search.proto',
  package='clarifai.api',
  syntax='proto3',
  serialized_pb=_b('\n\x1fproto/clarifai/api/search.proto\x12\x0c\x63larifai.api\x1a\x1fproto/clarifai/api/common.proto\x1a\x1eproto/clarifai/api/input.proto\x1a\x1fproto/clarifai/api/output.proto\x1a&proto/clarifai/api/status/status.proto\x1a)proto/clarifai/api/utils/extensions.proto\x1a\x30proto/clarifai/utils/pagination/pagination.proto\"8\n\x03Hit\x12\r\n\x05score\x18\x01 \x01(\x02\x12\"\n\x05input\x18\x02 \x01(\x0b\x32\x13.clarifai.api.Input\"_\n\x03\x41nd\x12\"\n\x05input\x18\x01 \x01(\x0b\x32\x13.clarifai.api.Input\x12$\n\x06output\x18\x02 \x01(\x0b\x32\x14.clarifai.api.Output\x12\x0e\n\x06negate\x18\x03 \x01(\x08\":\n\x05Query\x12\x1f\n\x04\x61nds\x18\x01 \x03(\x0b\x32\x11.clarifai.api.And\x12\x10\n\x08language\x18\x02 \x01(\t\"\xa0\x01\n\x13PostSearchesRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\"\n\x05query\x18\x02 \x01(\x0b\x32\x13.clarifai.api.Query\x12\x1f\n\npagination\x18\x03 \x01(\x0b\x32\x0b.Pagination\x12\x13\n\x0b\x62rute_force\x18\x04 \x01(\x08\"p\n\x19PostSearchFeedbackRequest\x12/\n\x0buser_app_id\x18\x01 \x01(\x0b\x32\x1a.clarifai.api.UserAppIDSet\x12\"\n\x05input\x18\x02 \x01(\x0b\x32\x13.clarifai.api.Input\"u\n\x13MultiSearchResponse\x12+\n\x06status\x18\x01 \x01(\x0b\x32\x1b.clarifai.api.status.Status\x12\n\n\x02id\x18\x02 \x01(\t\x12%\n\x04hits\x18\x03 \x03(\x0b\x32\x11.clarifai.api.HitB\x04\x80\xb5\x18\x01\x42$Z\x03\x61pi\xa2\x02\x04\x43\x41IP\xc2\x02\x01_\xca\x02\x11\x43larifai\\Internalb\x06proto3')
  ,
  dependencies=[proto_dot_clarifai_dot_api_dot_common__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_input__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_output__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_status_dot_status__pb2.DESCRIPTOR,proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2.DESCRIPTOR,proto_dot_clarifai_dot_utils_dot_pagination_dot_pagination__pb2.DESCRIPTOR,])




_HIT = _descriptor.Descriptor(
  name='Hit',
  full_name='clarifai.api.Hit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='score', full_name='clarifai.api.Hit.score', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input', full_name='clarifai.api.Hit.input', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=280,
  serialized_end=336,
)


_AND = _descriptor.Descriptor(
  name='And',
  full_name='clarifai.api.And',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='clarifai.api.And.input', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='output', full_name='clarifai.api.And.output', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='negate', full_name='clarifai.api.And.negate', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=338,
  serialized_end=433,
)


_QUERY = _descriptor.Descriptor(
  name='Query',
  full_name='clarifai.api.Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ands', full_name='clarifai.api.Query.ands', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='language', full_name='clarifai.api.Query.language', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=435,
  serialized_end=493,
)


_POSTSEARCHESREQUEST = _descriptor.Descriptor(
  name='PostSearchesRequest',
  full_name='clarifai.api.PostSearchesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.PostSearchesRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='query', full_name='clarifai.api.PostSearchesRequest.query', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pagination', full_name='clarifai.api.PostSearchesRequest.pagination', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='brute_force', full_name='clarifai.api.PostSearchesRequest.brute_force', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=496,
  serialized_end=656,
)


_POSTSEARCHFEEDBACKREQUEST = _descriptor.Descriptor(
  name='PostSearchFeedbackRequest',
  full_name='clarifai.api.PostSearchFeedbackRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_app_id', full_name='clarifai.api.PostSearchFeedbackRequest.user_app_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input', full_name='clarifai.api.PostSearchFeedbackRequest.input', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=658,
  serialized_end=770,
)


_MULTISEARCHRESPONSE = _descriptor.Descriptor(
  name='MultiSearchResponse',
  full_name='clarifai.api.MultiSearchResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='clarifai.api.MultiSearchResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='id', full_name='clarifai.api.MultiSearchResponse.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hits', full_name='clarifai.api.MultiSearchResponse.hits', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=772,
  serialized_end=889,
)

_HIT.fields_by_name['input'].message_type = proto_dot_clarifai_dot_api_dot_input__pb2._INPUT
_AND.fields_by_name['input'].message_type = proto_dot_clarifai_dot_api_dot_input__pb2._INPUT
_AND.fields_by_name['output'].message_type = proto_dot_clarifai_dot_api_dot_output__pb2._OUTPUT
_QUERY.fields_by_name['ands'].message_type = _AND
_POSTSEARCHESREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_POSTSEARCHESREQUEST.fields_by_name['query'].message_type = _QUERY
_POSTSEARCHESREQUEST.fields_by_name['pagination'].message_type = proto_dot_clarifai_dot_utils_dot_pagination_dot_pagination__pb2._PAGINATION
_POSTSEARCHFEEDBACKREQUEST.fields_by_name['user_app_id'].message_type = proto_dot_clarifai_dot_api_dot_common__pb2._USERAPPIDSET
_POSTSEARCHFEEDBACKREQUEST.fields_by_name['input'].message_type = proto_dot_clarifai_dot_api_dot_input__pb2._INPUT
_MULTISEARCHRESPONSE.fields_by_name['status'].message_type = proto_dot_clarifai_dot_api_dot_status_dot_status__pb2._STATUS
_MULTISEARCHRESPONSE.fields_by_name['hits'].message_type = _HIT
DESCRIPTOR.message_types_by_name['Hit'] = _HIT
DESCRIPTOR.message_types_by_name['And'] = _AND
DESCRIPTOR.message_types_by_name['Query'] = _QUERY
DESCRIPTOR.message_types_by_name['PostSearchesRequest'] = _POSTSEARCHESREQUEST
DESCRIPTOR.message_types_by_name['PostSearchFeedbackRequest'] = _POSTSEARCHFEEDBACKREQUEST
DESCRIPTOR.message_types_by_name['MultiSearchResponse'] = _MULTISEARCHRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Hit = _reflection.GeneratedProtocolMessageType('Hit', (_message.Message,), dict(
  DESCRIPTOR = _HIT,
  __module__ = 'proto.clarifai.api.search_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.Hit)
  ))
_sym_db.RegisterMessage(Hit)

And = _reflection.GeneratedProtocolMessageType('And', (_message.Message,), dict(
  DESCRIPTOR = _AND,
  __module__ = 'proto.clarifai.api.search_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.And)
  ))
_sym_db.RegisterMessage(And)

Query = _reflection.GeneratedProtocolMessageType('Query', (_message.Message,), dict(
  DESCRIPTOR = _QUERY,
  __module__ = 'proto.clarifai.api.search_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.Query)
  ))
_sym_db.RegisterMessage(Query)

PostSearchesRequest = _reflection.GeneratedProtocolMessageType('PostSearchesRequest', (_message.Message,), dict(
  DESCRIPTOR = _POSTSEARCHESREQUEST,
  __module__ = 'proto.clarifai.api.search_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.PostSearchesRequest)
  ))
_sym_db.RegisterMessage(PostSearchesRequest)

PostSearchFeedbackRequest = _reflection.GeneratedProtocolMessageType('PostSearchFeedbackRequest', (_message.Message,), dict(
  DESCRIPTOR = _POSTSEARCHFEEDBACKREQUEST,
  __module__ = 'proto.clarifai.api.search_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.PostSearchFeedbackRequest)
  ))
_sym_db.RegisterMessage(PostSearchFeedbackRequest)

MultiSearchResponse = _reflection.GeneratedProtocolMessageType('MultiSearchResponse', (_message.Message,), dict(
  DESCRIPTOR = _MULTISEARCHRESPONSE,
  __module__ = 'proto.clarifai.api.search_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.MultiSearchResponse)
  ))
_sym_db.RegisterMessage(MultiSearchResponse)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z\003api\242\002\004CAIP\302\002\001_\312\002\021Clarifai\\Internal'))
_MULTISEARCHRESPONSE.fields_by_name['hits'].has_options = True
_MULTISEARCHRESPONSE.fields_by_name['hits']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001'))
# @@protoc_insertion_point(module_scope)



"""
Geo support for clarifai api
"""


class GeoPoint(object):
  """ define a Geo point
      which is a (longitude, latitude) tuple
  """

  def __init__(self, longitude, latitude):
    self.longitude = float(longitude)
    self.latitude = float(latitude)

  def dict(self):
    return {'geo_point': {'longitude': self.longitude, 'latitude': self.latitude}}


class GeoBox(object):
  """ define a Geo box
      which is defined with a pair of GeoPoint
      representing the two corners of the Geo box
  """

  def __init__(self, point1, point2):
    self.point1 = point1
    self.point2 = point2

  def dict(self):
    return {'geo_box': [self.point1.dict(), self.point2.dict()]}


class GeoLimit(object):
  convert_table = {
      'mile': 'withinMiles',
      'kilometer': 'withinKilometers',
      'degree': 'withinDegrees',
      'radian': 'withinRadians'
  }

  def __init__(self, limit_type='mile', limit_range=10):
    if limit_type not in self.convert_table:
      raise ValueError("limit_type could be within %s" % str(self.convert_table.keys()))

    self.limit_type = self.convert_table[limit_type]
    self.limit_range = float(limit_range)

  def dict(self):
    return {'geo_limit': {'type': self.limit_type, 'value': self.limit_range}}


class Geo(object):

  def __init__(self, geo_point=None, geo_limit=None, geo_box=None):

    self.geo_point = geo_point
    self.geo_limit = geo_limit
    self.geo_box = geo_box

    # only geo_point for input
    if geo_point is not None and geo_limit is None and geo_box is None:
      pass
    # geo_point and geo_limit for search within range
    elif geo_point is not None and geo_limit is not None and geo_box is None:
      pass
    # only geo_box for search
    elif geo_point is None and geo_limit is None and geo_box is not None:
      pass
    else:
      raise Exception('Invalid Geo object initialization')

  def dict(self):

    # only geo_point for input
    if self.geo_point is not None and self.geo_limit is None and self.geo_box is None:
      data = {'geo': self.geo_point.dict()}
    # geo_point and geo_limit for search within range
    elif self.geo_point is not None and self.geo_limit is not None and self.geo_box is None:
      data = {'geo': self.geo_point.dict()}
      data['geo'].update(self.geo_limit.dict())
    # only geo_box for search
    elif self.geo_point is None and self.geo_limit is None and self.geo_box is not None:
      data = {'geo': self.geo_box.dict()}
    else:
      raise Exception('Invalid Geo object')

    return data
